<?php
/*
	Plugin Name: Cloudflare for WP Cerber Security
	Plugin URI: https://wpcerber.com
	Description: This is a Cloudflare integration add-on for WP Cerber Security. It provides continuous synchronization Cloudflare IP Access Rules with IP addresses blocked by WP Cerber as well as IP addresses in the WP Cerber's IP Access Lists. Requires the WP Cerber Security plugin 9.6.3 or newer.
	Author: Cerber Tech Inc.
	Author URI: https://wpcerber.com
	Version: 1.3
	Requires PHP: 7.2
	Requires at least: 5.8
	Text Domain: wp-cerber-cloudflare-addon
	Domain Path: /languages
	Network: true

	Copyright (C) 2019-25 CERBER TECH INC., https://cerber.tech
	Copyright (C) 2019-25 CERBER TECH INC., https://wpcerber.com

    Licenced under the GNU GPL.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

if ( ! defined( 'CERBER_VER' ) || ! function_exists( 'cerber_register_addon' ) ) {
	return;
}

const CRB_CLF_PLID = 'wp-cerber-cloudflare-addon/wp-cerber-cloudflare.php';
const CRB_CLF_PREFIX = 'CLF_RID_';

cerber_register_addon( __FILE__, 'cloudflare', 'Cloudflare', '9.6.3', 'clf_addon_settings', '_cfl_check_settings' );
cerber_add_handler( 'ip_event', 'clf_addon_process_ip', 'cloudflare' );
cerber_add_handler( 'deactivated', '_clf_clear_up', 'cloudflare' );

/**
 * @return array[]
 *
 * @since 1.3
 */
function clf_addon_settings(): array {
	return array(
		'cf_main' => array(
			'name'    => __( 'Cloudflare firewall rules syncing', 'wp-cerber-cloudflare-addon' ),
			'desc'    => __( 'If enabled, those features add and remove IP addresses to and from the Cloudflare IP Access Rules continuously', 'wp-cerber-cloudflare-addon' ),
			'doclink' => 'https://wpcerber.com/cloudflare-add-on-wp-cerber/',
			'fields'  => array(
				'cloudflare_lockout' => array(
					'title' => __( 'Enable lockout syncing', 'wp-cerber-cloudflare-addon' ),
					//'label' => __( 'Sync', 'wp-cerber-cloudflare-addon' ),
					'type'  => 'checkbox',
				),
				'cloudflare_acl'     => array(
					'title' => __( 'Enable ACL syncing', 'wp-cerber-cloudflare-addon' ),
					'type'  => 'checkbox',
				),
				'cloudflare_verb'    => array(
					'title' => __( 'Verbose syncing', 'wp-cerber-cloudflare-addon' ),
					'type'  => 'checkbox',
				),
				'cloudflare_del'     => array(
					'title' => __( 'Delete Cloudflare rules on plugin deactivation', 'wp-cerber-cloudflare-addon' ),
					'type'  => 'checkbox',
				),
				'cloudflare_log'   => array(
					'title' => __( 'Enable diagnostic logging', 'wp-cerber-cloudflare-addon' ),
					'label' => 'Once enabled, the log is available here: <a target="_blank" href="' . cerber_admin_link( 'diag-log' ) . '">Diagnostic Log</a>',
					'type'  => 'checkbox',
				),
				'cloudflare_email' => array(
					'title'       => __( 'Email Address', 'wp-cerber-cloudflare-addon' ),
					'placeholder' => 'Your Cloudflare account email',
					'type'        => 'email',
				),
				'cloudflare_key'   => array(
					'title'       => __( 'Global API Key', 'wp-cerber-cloudflare-addon' ),
					'placeholder' => 'Your Cloudflare API Key',
				),
			),
		),
	);
}

function clf_addon_process_ip( $data = array() ) {
	global $cloudflare_error;

	switch ( $data['e_type'] ) {
		case 'locked':
			if ( ! clf_addon_get_setting( 'cloudflare_lockout' )
			     || cerber_check_groove()
			     || $data['reason_id'] == 711
			     || ( ! $data['update'] && in_array( $data['reason_id'], array( 701, 703 ) ) )
			     || _cfl_is_google_bot( $data['ip'] ) ) {

				return;
			}
			if ( ! cerber_is_ip( $data['ip'] ) ) {
				_cfl_log( 'Subnet-wide lockouts are not supported: ' . $data['ip'] );

				return;
			}

			_clf_add_rule( $data['ip'], 'block', $data['reason'] );

			break;

		case 'unlocked':
			if ( $data['result'] ) {
				_clf_delete_rule( $data['ip'] );
			}
			else {
				_cfl_log( 'ERROR: Addresses were not completely deleted from Cerber\'s lockout table' );
			}
			break;

		case 'acl_add':
			if ( $data['slice'] !== 0
			     || ! clf_addon_get_setting( 'cloudflare_acl' ) ) {
				return;
			}

			if ( $data['tag'] == 'B' ) {
				$result = _clf_add_rule( $data['ip'], 'block', $data['comments'] );
			}
			elseif ( $data['tag'] == 'W' ) {
				$result = _clf_add_rule( $data['ip'], 'whitelist', $data['comments'] );
			}

			if ( ! $result ) {
				cerber_admin_notice( 'Unable to add a Cloudflare rule. ' . $cloudflare_error );
			}

			break;

		case 'acl_remove':
			if ( $data['slice'] !== 0
			     || ! clf_addon_get_setting( 'cloudflare_acl' ) ) {
				return;
			}
			_clf_delete_rule( $data['ip'] );
			break;
	}
}

/**
 * @param string $ip
 * @param string $mode 'block' or 'whitelist'
 * @param string $reason
 *
 * @return bool
 */
function _clf_add_rule( $ip, $mode = 'block', $reason = '' ) {

	$key = CRB_CLF_PREFIX . $ip;
	if ( cerber_get_set( $key, null, false ) ) {
		return true;
	}

	$rule_id = cloudflare_api_send( array( 'ip' => $ip, 'mode' => $mode, 'reason' => $reason ) );

	if ( $rule_id ) {
		cerber_update_set( CRB_CLF_PREFIX . $ip, $rule_id, null, false );
		_cfl_log( 'Add ' . $mode . ' rule for ' . $ip . ' / ' . $reason );

		return true;
	}

	_cfl_log( 'ERROR: Unable to add rule for ' . $ip . ' / ' . $reason );

	return false;

}

/**
 * @param string|array $ip
 *
 * @return bool
 */
function _clf_delete_rule( $ip ) {
	if ( ! is_array( $ip ) ) {
		$ip = array( $ip );
	}

	$ret = true;

	foreach ( $ip as $item ) {
		$key = CRB_CLF_PREFIX . $item;
		if ( ! $entry = cerber_get_set( $key, null, false ) ) {
			continue;
		}

		if ( cloudflare_api_send( array( 'rule_id' => $entry ), true ) ) {
			cerber_delete_set( $key );
			_cfl_log( 'Delete rule for IP ' . $item );
		}
		else {
			_cfl_log( 'ERROR: Unable to unlock IP ' . $item );
			$ret = false;
		}
	}

	return $ret;
}

/**
 * Manage Cloudflare IP Access Rules
 *
 * @param array $data
 * @param bool $delete
 *
 * @return bool
 */
function cloudflare_api_send( $data, $delete = false ) {
	global $cloudflare_error;

	$cloudflare_error = '';
	$email = clf_addon_get_setting( 'cloudflare_email' );
	$key = clf_addon_get_setting( 'cloudflare_key' );
	$verbose = clf_addon_get_setting( 'cloudflare_verb' );

	if ( ! $email || ! $key ) {
		return false;
	}

	$headers = array(
		'Content-Type: application/json',
		'X-Auth-Email: ' . $email,
		'X-Auth-Key: ' . $key
	);

	$curl_opt = array(
		CURLOPT_RETURNTRANSFER => true,
		CURLOPT_HTTPHEADER     => $headers,
	);

	if ( $delete ) {
		$curl_opt[ CURLOPT_CUSTOMREQUEST ] = 'DELETE';
		$curl_opt[ CURLOPT_URL ] = 'https://api.cloudflare.com/client/v4/user/firewall/access_rules/rules/' . $data['rule_id'];
	}
	else {
		$note = '';

		if ( $data['mode'] == 'block' ) {
			$note = 'Blocked';
		}
		elseif ( $data['mode'] == 'whitelist' ) {
			$note = 'Whitelisted';
		}
		$note .= ' by WP Cerber on ' . cerber_date( time() );

		if ( $verbose ) {
			$note .= ' / ' . $data['reason'];
		}

		$config = array( 'value' => $data['ip'] );

		if ( cerber_is_ip( $data['ip'] ) ) {
			$config['target'] = 'ip'; // Single IP
		}
		else {
			$config['target'] = 'ip_range'; // CIDR
		}

		$clf_data = array(
			'mode'          => $data['mode'], // can be 'block' or 'whitelist'
			'configuration' => $config,
			'notes'         => $note,
		);

		if ( ! $json = json_encode( $clf_data ) ) {
			return false;
		}

		$curl_opt[ CURLOPT_POSTFIELDS ] = $json;
		$curl_opt[ CURLOPT_URL ]        = 'https://api.cloudflare.com/client/v4/user/firewall/access_rules/rules';
	}

	$curl = curl_init();
	curl_setopt_array( $curl, $curl_opt );
	$result = curl_exec( $curl );
	$ret = false;

	if ( $result == false ) {
		$curl_error = curl_error( $curl );

		_cfl_log( 'ERROR: CURL ' . curl_errno( $curl ) . ': ' . $curl_error );

	}
	else {
		$result = json_decode( $result, true );

		//_cfl_log( 'RESPONSE ' . print_r( $result, 1 ) );

		if ( ! empty( $result['success'] )
		     && isset( $result['result']['id'] ) ) {
			$ret = $result['result']['id'];
		}
		elseif ( ! empty( $result['errors'] ) ) {
			foreach ( $result['errors'] as $error ) {
				_cfl_log( 'ERROR: Cloudflare API / ' . $error['message'] );
				$cloudflare_error .= $error['message'] . ' ';
			}
		}
	}

	curl_close( $curl );
	return $ret;
}

/**
 * Return true if $ip is Googlebot IP: avoid blocking Google crawlers.
 *
 * @param string $ip
 *
 * @return bool
 */
function _cfl_is_google_bot( $ip ) {
	if ( ! cerber_is_ip( $ip )
	     || empty( $_SERVER['HTTP_USER_AGENT'] )
	     || ! strpos( $_SERVER['HTTP_USER_AGENT'], 'Googlebot' ) ) {
		return false;
	}

	if ( ! get_option( 'blog_public' ) ) {
		return false;
	}

	$hostname = @gethostbyaddr( $ip );

	if ( $hostname
	     && ( substr( $hostname, - 14 ) === '.googlebot.com' ) ) {
		return true;
	}

	return false;
}

function _cfl_check_settings( $values = array() ) {
	if ( ! _clf_add_rule( '10.254.254.254', 'whitelist', 'Test rule by WP Cerber' ) ) {
		cerber_admin_notice( 'Unable to verify Cloudflare credentials' );

		return;
	}
	if ( ! _clf_delete_rule( '10.254.254.254' ) ) {
		cerber_admin_notice( 'Unable to verify Cloudflare credentials' );

		return;
	}

	if ( clf_addon_get_setting( 'subnet' ) && $values['cloudflare_lockout'] ) {
		cerber_admin_notice( 'Lockout syncing for subnet-wide lockouts are not supported. Please read the documentation.' );
	}
}

function _cfl_log( $msg ) {
	if ( clf_addon_get_setting( 'cloudflare_log' ) ) {
		cerber_diag_log( $msg, 'Cloudflare Addon' );
	}
}

/**
 * Remove all Cloudflare rules when the plugin has been deactivated
 *
 */
if ( clf_addon_get_setting( 'cloudflare_del' ) ) {
	add_action( 'deactivate_' . CRB_CLF_PLID, '_clf_clear_up' );
}
function _clf_clear_up() {
	if ( ! clf_addon_get_setting( 'cloudflare_del' ) ) {
		return;
	}
	if ( ! $list = cerber_db_get_results( 'SELECT * FROM ' . cerber_get_db_prefix() . CERBER_SETS_TABLE . ' WHERE the_key LIKE "' . CRB_CLF_PREFIX . '%"' ) ) {
		return;
	}
	foreach ( $list as $entry ) {
		cloudflare_api_send( array( 'rule_id' => $entry['the_value'] ), true );
		cerber_delete_set( $entry['the_key'] );
	}
}

/**
 * @param $id
 *
 * @return mixed
 *
 * @since 1.2
 */
function clf_addon_get_setting( $id = '' ) {

	// Before WP Cerber 9.0
	if ( function_exists( 'cerber_get_settings' ) ) {
		return cerber_get_settings( $id );
	}

	// WP Cerber 9.0 and newer
	return crb_get_settings( $id );
}